-- BUGFIX verso i ~230m c'è un bug di un collider
-- BUGFIX a volte rimane appeso ad un gradino
-- TODO mettere un falò, molto importante
-- TODO opzionale, mettere delle spine che salgono
-- TODO opzionale, mettere delle palle di fuoco
-- TODO powerup #1 invincibilità ad un danno
-- TODO powerup #2 grip temporaneo

function love.load()
    love.window.setMode(1000, love.graphics.getHeight()+140)


    require('gamemode')
    
    loadGameMode()

    math.randomseed(os.time())

    debugFont = love.graphics.newFont(10)
    gameFont = love.graphics.newFont(20)

    isPaused = false
    debugMode = false

    levelZero = 0

    anim8 = require './libraries/anim8/anim8'
    cameraFile = require './libraries/hump/camera'

    cam = cameraFile()

    sprites = {}
    sprites.playerSheetIdle = love.graphics.newImage('sprites/knight/_Idle.png')
    sprites.playerSheetJump = love.graphics.newImage('sprites/knight/_Jump.png')
    sprites.playerSheetRun = love.graphics.newImage('sprites/knight/_Run.png')
    sprites.playerSheetDead = love.graphics.newImage('sprites/knight/_Death.png')
    sprites.playerSheetWallSlide = love.graphics.newImage('sprites/knight/_WallSlide.png')
    sprites.wall = love.graphics.newImage('sprites/Castle_Wall.png')
    sprites.spear = love.graphics.newImage('sprites/spear.png')
    sprites.torch = love.graphics.newImage('sprites/torch.png')

    local gridIdle = anim8.newGrid(120, 80, sprites.playerSheetIdle:getWidth(), sprites.playerSheetIdle:getHeight())
    local gridJump = anim8.newGrid(120, 80, sprites.playerSheetJump:getWidth(), sprites.playerSheetJump:getHeight())
    local gridRun = anim8.newGrid(120, 80, sprites.playerSheetRun:getWidth(), sprites.playerSheetRun:getHeight())
    local gridDead = anim8.newGrid(120, 80, sprites.playerSheetDead:getWidth(), sprites.playerSheetDead:getHeight())
    local gridWallSlide = anim8.newGrid(120, 80, sprites.playerSheetWallSlide:getWidth(), sprites.playerSheetWallSlide:getHeight())

    animations = {}
    animations.idle = anim8.newAnimation(gridIdle('1-10',1), 0.15)
    animations.jump = anim8.newAnimation(gridJump('1-3',1), 0.2)
    animations.run = anim8.newAnimation(gridRun('1-10',1), 0.15)
    animations.dead = anim8.newAnimation(gridDead('1-10',1), 0.15)
    animations.wallSlide = anim8.newAnimation(gridWallSlide('1-3',1), 0.15)

    defaultGravity = 3000

    wf = require 'libraries/windfield/windfield'
    world = wf.newWorld(0, defaultGravity, false)

    dangers = {}

    world:addCollisionClass('Floor')
    world:addCollisionClass('WallR')
    world:addCollisionClass('WallL')
    world:addCollisionClass('Danger')
    world:addCollisionClass('Player')--, {pre = {'WallR', 'WallL'}}
    world:addCollisionClass('Dead', {ignores = {'Danger'}})
    world:addCollisionClass('Ghost', {ignores = {'Danger', 'WallR', 'WallL'}})

    require('player')
    require('walls')

    loadWalls(levelZero)

    local floor = world:newRectangleCollider(-love.graphics.getWidth()*5, levelZero, love.graphics.getWidth()*10, 1, {collision_class="Floor"})
    floor:setType('static')
    
    score = 0

end

function spawnSpears(topX, topY)

    local spear = world:newPolygonCollider({topX, topY, topX+50, topY-10, topX, topY-20}, {collision_class='Danger'})
    spear:setType('static')

end

function reset()
    resetPlayerPosition()
    resetAssets()
    GameMode:next()
    score = 0
end

function love.keyreleased(key)

    if key == 'space' or key == 'up' then
        local vx, vy = player:getLinearVelocity()
        player:setLinearVelocity(vx, vy/2)
    end

end

function love.keypressed(key)


    if key == 'x' then
        GameMode:toggleCreative()
    end

--[[     if key == 'm' then
        for k, obj in pairs(assets) do
            obj:setPosition(obj:getX(), obj:getY()+100)
        end
        for k, obj in pairs(spears) do
            if obj ~= nil and type(obj) ~= 'function' then
                obj:translateVertically(100)
            end
        end
    end
 ]]
    if key == 'return' then
        GameMode:start()
    end

    if key == 'q' then
        love.event.quit()
    end

    if key == 'b' then
        debugMode = not debugMode
    end
    world:setQueryDebugDrawing(debugMode)

    if (GameMode.Play or GameMode.Pause) and key == 'p' then
        isPaused = not isPaused
        GameMode:togglePause()
    end

    if GameMode.Play and not player.isDead and isPlayerTouchingWall() and (key == 'space' or key == 'up') then

        local lateralJump = 0
        local verticalJump = -5000

        if isPlayerTouchingWall() then
            lateralJump = 5500
            verticalJump = -9000
        end

        player:applyLinearImpulse(lateralJump * player.direction, verticalJump)
    elseif player.isDead and (key == 'space' or key == 'up') then
        reset()
    end
end

function getScore()
    return math.max(score, math.ceil(-1 * player:getY() / 100))
end

function love.update(dt)

    if not GameMode.Pause then

        world:update(dt)

        score = getScore()
    
        local newGravity = defaultGravity
    
        if isPlayerTouchingWall() then
    
            local vx, vy = player:getLinearVelocity()
    
            if vy < 0 then
                player:setLinearVelocity(0, 0)
            end
    
    
            if love.keyboard.isDown('r') and isPlayerFalling() then
                newGravity = defaultGravity*0.01
            else
                newGravity = defaultGravity*0.3
            end
        end
    
        if GameMode.Creative then
            newGravity = 0
        end

        world:setGravity(0, newGravity) 
    
    
        playerUpdate(dt)
    
        local px, py = player:getPosition()
        local cameraLevel = py
    
        cam:lookAt(360, cameraLevel)
    
        updateWalls(player)

    end

    

end

function love.draw()



    cam:attach()

        drawWalls()
        playerDraw()

        if debugMode then
            world:draw()
        end

    cam:detach()
    local px, py = player:getPosition()

    love.graphics.setFont(gameFont)

    love.graphics.printf(score .. "m", 0, 10, love.graphics.getWidth(), "center")

    love.graphics.setFont(debugFont)
    
    love.graphics.printf("GameMode[Start] = " .. tostring(GameMode.Start), 0, 90, love.graphics.getWidth(), "center")
    love.graphics.printf("GameMode[Pause] = " .. tostring(GameMode.Pause), 0, 80, love.graphics.getWidth(), "center")
    love.graphics.printf("GameMode[Play] = " .. tostring(GameMode.Play), 0, 70, love.graphics.getWidth(), "center")
    love.graphics.printf("GameMode[GameOver] = " .. tostring(GameMode.GameOver), 0, 60, love.graphics.getWidth(), "center")
    love.graphics.printf("GameMode[Creative] = " .. tostring(GameMode.Creative), 0, 50, love.graphics.getWidth(), "center")
    love.graphics.printf("Player pos " .. player:getX() .. " " .. player:getY(), 0, 40, love.graphics.getWidth(), "center")
    love.graphics.printf("# assets " .. #assets, 0, 30, love.graphics.getWidth(), "center")



    GameMode:draw()


--[[ 
    love.graphics.printf("mindistanceBetweenSpears: " .. mindistanceBetweenSpears, 0, love.graphics.getHeight() - 100, love.graphics.getWidth(), "center")

    love.graphics.print("Player speed right: " .. player.rightSpeed, 700, 35)
    love.graphics.print("Player speed left: " .. player.leftSpeed, 700, 65)
    love.graphics.print("Player is touching L wall: " .. tostring(isTouchingWallLeft()), 700, 95)
    love.graphics.print("Player is touching R wall: " .. tostring(isTouchingWallRight()), 700, 125)
    love.graphics.print("Player is touching any wall: " .. tostring(isPlayerTouchingWall()), 700, 155)
    love.graphics.print("Player direction: " .. tostring(player.direction), 700, 185)
    love.graphics.print("Player isJumping: " .. tostring(isPlayerJumping()), 700, 215)
    love.graphics.print("Player isWayUpperVoid: " .. tostring(isWayUpperVoid(heightBlock)), 700, 245)
    --love.graphics.print("Player isFalling: " .. tostring(isPlayerFalling()), 700, 275)
    love.graphics.print("Player maxH: " .. findMaxHeightAsset(), 700, 280)
    love.graphics.print("#assets: " ..  #assets, 700, 305) ]]


end