playerStartX = 200 -- TODO metterlo in config comuni globali
playerStartY = -50
playerBoxWidth = 50
playerBoxHeight = 50

player = world:newRectangleCollider(playerStartX, playerStartY, playerBoxWidth, playerBoxHeight, {collision_class="Player"})
player:setFixedRotation(true)
player.rightSpeed = 500
player.leftSpeed = 500
player.direction = 1
player.animation = animations.idle
player.isRunning = false
player.isDead = false
player.storeVx = nil
player.storeVy = nil

require("gamemode")

function playerUpdate(dt)

    if player:enter("Danger") then
        player.isDead = true
        GameMode:gameover()
    end

    if player:enter("WallR") then
        player.rightSpeed = 0
        player.direction = -1
    elseif player:enter("WallL") then
        player.leftSpeed = 0
        player.direction = 1
    elseif player:exit("WallR") then
        player.rightSpeed = 500
    elseif player:exit("WallL") then
        player.leftSpeed = 500
    end

    local px, py = player:getPosition()

    if not player.isDead and GameMode.Play then

        if isPlayerGrounded() then
            if love.keyboard.isDown('right') then
                player:setX(px + player.rightSpeed * dt)
                player.direction = -1
                player.isRunning = true
            elseif love.keyboard.isDown('left') then
                player:setX(px - player.leftSpeed * dt)
                player.direction = 1
                player.isRunning = true
            else
                player.isRunning = false
            end
        else
            player.isRunning = false
        end

        if isPlayerJumping() then
            player.animation = animations.jump
        elseif player.isRunning then
            player.animation = animations.run
        elseif isPlayerGrounded() then
            player.animation = animations.idle
        elseif isPlayerTouchingWall() then
            player.animation = animations.wallSlide
        end

    elseif GameMode.Creative then
        if love.keyboard.isDown('right') then
            player:setX(px + player.rightSpeed * 10 * dt)
        end
        if love.keyboard.isDown('up') then
            player:setY(py - player.rightSpeed * 10 * dt)
        end
        if love.keyboard.isDown('left') then
            player:setX(px - player.rightSpeed * dt)
        end
        if love.keyboard.isDown('down') then
            player:setY(py + player.rightSpeed * dt)
        end

    else
        player:setCollisionClass('Dead')

        player.animation = animations.dead
        player.animation:pauseAtEnd()

        local vx, vy = player:getLinearVelocity()
        player:setLinearVelocity(0, vy)
    end




    player.animation:update(dt)

end

function playerPause()
    local vx, vy = player:getLinearVelocity()
    player.storeVx = vx
    player.storeVy = vy
    player:setType('static')
    player.animation:pause()
end

function playerResume()
    player.animation:resume()
    player:setType('dynamic')
    player:setLinearVelocity(player.storeVx, player.storeVy)
end

function playerDraw()
    local px, py = player:getPosition()
    if player.isDead then
        player.animation:draw(sprites.playerSheetDead, px, py, nil, 1.5 * -player.direction, 1.5, 55, 65)
    elseif GameMode.Creative then
        player.animation:draw(sprites.playerSheetIdle, px, py, nil, 1.5 * -player.direction, 1.5, 55, 65)
    elseif isPlayerJumping() then
        player.animation:draw(sprites.playerSheetJump, px, py, nil, 1.5 * -player.direction, 1.5, 55, 65)
    elseif player.isRunning then
        player.animation:draw(sprites.playerSheetRun, px, py, nil, 1.5 * -player.direction, 1.5, 65, 65)
    elseif isPlayerGrounded() then
        player.animation:draw(sprites.playerSheetIdle, px, py, nil, 1.5 * -player.direction, 1.5, 55, 65)
    elseif isPlayerTouchingWall() then
        player.animation:draw(sprites.playerSheetWallSlide, px, py, nil, 1.5 * player.direction, 1.5, 65, 65)
    end
end

function isPlayerFalling()
    local vx, vy = player:getLinearVelocity()
    return  vy > 0
end

function resetPlayerPosition()
    player:setPosition(playerStartX, playerStartY)
    player.isDead = false
    player.rightSpeed = 500
    player.leftSpeed = 500
end

function isTouchingWallLeft()
    local colliders = world:queryRectangleArea(player:getX() - playerBoxWidth/2 - 1, player:getY() - playerBoxWidth/2 - 1, 1, playerBoxWidth, {'WallL'})
    return #colliders > 0
end

function isTouchingWallRight()
    local colliders = world:queryRectangleArea(player:getX() + playerBoxWidth/2, player:getY() - playerBoxWidth/2 - 1, 1, playerBoxWidth, {'WallR'})
    return #colliders > 0
end

function isPlayerTouchingWall()
    return isTouchingWallLeft() or isTouchingWallRight()
end

function isPlayerJumping()
    return not isPlayerTouchingWall() and not isPlayerGrounded()
end

function isPlayerGrounded()
    local colliders = world:queryRectangleArea(player:getX() - 20, player:getY()+25, 40, 2, {'Floor'})
    return #colliders > 0
end

function isWayUpperVoid(distance)
    local colliders = world:queryRectangleArea(
        player:getX()-love.graphics.getWidth(),
        player:getY()-math.abs(distance),
        player:getX()+love.graphics.getWidth(),
        1
    )
    return #colliders == 0
end

function getBlockNumber(blockHeight)
    return player.getY() / blockHeight
end

