Spear = {}

function Spear:new(sx, sy, spearType)

    if spearType == nil then
        spearType = randomSpearType()
    end

    assertSpearType(spearType)

    local newObj = {sx = sx, sy = sy, spearType = spearType}
    self.__index = self
    local collL, collR = self:spawn(newObj)
    newObj.collL = collL
    newObj.collR = collR
    return setmetatable(newObj, self)

end

function Spear:translateVertically(y)
    if self.collL ~= nil then
        local newY = self.collL:getY()+y
        self.collL:setY(newY)
        self.sy = newY
    end
    if self.collR ~= nil then
        local newY = self.collR:getY()+y
        self.collR:setY(newY)
        self.sy = newY
    end
end

function Spear:destroy()
    if self.collL ~= nil then
        self.collL:destroy()
    end
    if self.collR ~= nill then
        self.collR:destroy()
    end
end

function randomSpearType()
    local rnd = math.random(0, 100)
    if rnd <= 40 then
        return 'R'
    elseif rnd <= 80 then
        return 'L'
    else
        return 'B'
    end
end

function Spear:spawn(spear)

    local spearType = spear.spearType

    if spearType == 'L' then
        return spawnLeftSpear(spear), nil
    elseif spearType == 'R' then
        return nil, spawnRightSpear(spear)
    else
        return spawnLeftSpear(spear), spawnRightSpear(spear)
    end
    
end

function Spear:draw(spear)

    local spearType = spear.spearType
    local sx = spear.sx
    local sy = spear.sy
    
    if spearType == 'L' then
        drawLeftSpear(spear)
    elseif spearType == 'R' then
        drawRightSpear(spear)
    else
        drawLeftSpear(spear)
        drawRightSpear(spear)
    end
    
end

function spawnLeftSpear(spear)
    local topX = spear.sx
    local topY = spear.sy
    local spearLeft= world:newPolygonCollider({topX, topY, topX+50, topY-10, topX, topY-20}, {collision_class='Danger'})
    spearLeft:setType('static')

    print("SPEARS X L " .. topX)
    print("SPEARS Y L " .. topY)

    return spearLeft
end

function drawLeftSpear(spear)
    love.graphics.draw(sprites.spear, 0+thicknessWall, spear.sy, nil, 2, nil, -10, 8)
end

function drawRightSpear(spear)
    love.graphics.draw(sprites.spear, 0+thicknessWall+distanceBetweenWall, spear.sy, nil, -2, nil, 40, -3)
end

function spawnRightSpear(spear)
    local topXRight = spear.sx + distanceBetweenWall
    local topY = spear.sy
    local spearRight= world:newPolygonCollider({topXRight, topY, topXRight-50, topY-10, topXRight, topY-20}, {collision_class='Danger'})
    spearRight:setType('static')


    print("SPEARS X R " .. topXRight)
    print("SPEARS Y R " .. topY)

    return spearRight
end

function assertSpearType(spearType)
    if spearType == nil then
        error("Speartype can't be null")
    end
    if spearType ~= 'L' and spearType ~= 'R' and spearType ~= 'B' then
         error("Speartype " .. spearType .. " not supported, choose one among L|R|B.")
    end
end